/**
 * @file cylinder.cpp
 * Implementation of Cylinder class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */

#include "cylinder.h"
#include "model.h"
#include "scene.h"

Cylinder::Cylinder( Model *model, Vec3 position, dMatrix3 orientation, double scale, double mass ) :
    Object( model, position, orientation, scale, mass )
{
    dMass massStruct;

    Vec3 params = model->get_max() - model->get_min();

    double radius = min( params[ 0 ], min( params[ 1 ], params[ 2 ] ) ) / 2.0;
    double length = max( params[ 0 ], max( params[ 1 ], params[ 2 ] ) ) - 2.0 * radius;

    dMassSetCappedCylinderTotal( &massStruct, mass, 3, radius, length );

    dBodySetMass( _body_id, &massStruct );

    _geom_id = dCreateCCylinder( get_space_id(), radius, length );

    dGeomSetBody( _geom_id, _body_id );

    dGeomSetPosition( _geom_id, position[ 0 ], position[ 1 ], position[ 2 ] );

    dGeomSetRotation( _geom_id, orientation );
}

Cylinder::~Cylinder( void )
{

}

void Cylinder::tick( double t )
{

}

EffectHandler Cylinder::get_effect_handler( Effect *effect )
{
    return Object::get_effect_handler( effect );
}
