/**
 * @file object.cpp
 * Implementation of Object class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */

#include "object.h"
//#include "physicaleffect.h"
#include "model.h"
#include "scene.h"

#include <gfx/gl.h>

Object::Object(Model *model, Vec3 position, dMatrix3 orientation, double scale, double mass ) :
    _model( model ),
    _scale( scale ),
    _stationary( false ),
    _ethereal( false ),
    _on( false )
{
    // just so lots of other functions dont crash
    if( !model ) {
        model = new Model();
    }

    _body_id = dBodyCreate( get_world_id() );

    dBodySetPosition( _body_id, position[ 0 ], position[ 1 ], position[ 2 ] );
    _sim_position = position;

    if( orientation == NULL ) {
        dRSetIdentity( orientation );
    }

    dBodySetRotation( _body_id, orientation );
    dMatrix3TodMatrix3( _sim_orientation, orientation );

    _sim_on = _on;
}

Object::~Object( void )
{
    // remove this object from ode
    dBodyDestroy( _body_id );
    dGeomDestroy( _geom_id );
}


EffectHandler Object::get_effect_handler( Effect *effect )
{
//    if ( IS_PHYSICAL_EFFECT( effect ) && !is_ethereal() ) {
//        return (EffectHandler)&Object::_handle_physical_effect;
//    }
    return NULL; // no handler for this effect
}

void Object::_handle_physical_effect( PhysicalEffect *effect )
{
    /** @FIXME I don't know how to do rotations. Figure out where the
     * effect is acting, calculate rotation, etc
     */
    /// @FIXME collisions?

    /*
    if ( !is_stationary() ) {
        _net_force += effect->get_force();
    }
    */
}

void Object::render( bool selected )
{
    glPushMatrix();

    /*
    glPushAttrib(GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);
    glColor3f(1.0f, 1.0f, 1.0f);
 
    glBegin(GL_LINES);
    glVertex3d( _position[0], _position[1], _position[2] );
    Vec3 tmp = _position + _velocity;
    glVertex3d( tmp[0], tmp[1], tmp[2] );
    glEnd();

    glPopAttrib();
    */

    Vec3 pos = dBodyGetPosition( _body_id );

    glTranslated( pos[ 0 ], pos[ 1 ], pos[ 2 ] );

    dMatrix3 odeRot;
    dRealPtrTodMatrix3( odeRot, dBodyGetRotation( _body_id ) );

    float oglRot[ 16 ];
    dMatrix3ToOpenGL( oglRot, odeRot );

    glMultMatrixf( oglRot );

    glScaled( _scale, _scale, _scale );

    // glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

    _model->render();

    if ( selected ) {
        _model->draw_selection();
    }

    glPopMatrix();
}

void Object::update_sim( void )
{
    dRealPtrToVec3( _sim_position, dBodyGetPosition( _body_id ) );

    dRealPtrTodMatrix3( _sim_orientation, dBodyGetRotation( _body_id ) );

    _sim_on = _on;
}

void Object::reset_sim( void )
{
    dBodySetPosition( _body_id, _sim_position[ 0 ], _sim_position[ 1 ], _sim_position[ 2 ] );

    dBodySetRotation( _body_id, _sim_orientation );

    _on = _sim_on;

    // are these necessary?
    dBodySetForce( _body_id, 0.0, 0.0, 0.0 );
    dBodySetTorque( _body_id, 0.0, 0.0, 0.0 );
}

// should not be necessary anymore
/*
void Object::update_pos(double t)
{
    if ( is_stationary() ) {
        return;
    }

    Vec3 accel = _net_force / _mass;
    _velocity += accel * t;
    _position += _velocity * t;

    _net_force = 0;

    /// @FIXME I don't know how to do rotations
//    _ang_velocity += _moment * _net_torque * t;
//    _rotation_angle += t;

    _net_torque = 0;

    if ( _position[1] <= -30.0 ) {
        _velocity[1] = 0.9*fabs(_velocity[1]);
    }
}
*/
