/**
 * @file sphere.cpp
 * Implementation of Sphere class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */

#include "sphere.h"
#include "model.h"
#include "scene.h"

Sphere::Sphere( Model *model, Vec3 position, dMatrix3 orientation, double scale, double mass ) :
    Object( model, position, orientation, scale, mass )
{
    dMass massStruct;

    Vec3 length = model->get_max() - model->get_min();

    double radius = min( length[ 0 ], min( length[ 1 ], length[ 2 ] ) ) / 2.0;

    dMassSetSphereTotal( &massStruct, mass, radius );

    dBodySetMass( _body_id, &massStruct );

    _geom_id = dCreateSphere( get_space_id(), radius );

    dGeomSetBody( _geom_id, _body_id );

    dGeomSetPosition( _geom_id, position[ 0 ], position[ 1 ], position[ 2 ] );

    dGeomSetRotation( _geom_id, orientation );
}

Sphere::~Sphere( void )
{

}

void Sphere::tick( double t )
{

}

EffectHandler Sphere::get_effect_handler( Effect *effect )
{
    return Object::get_effect_handler( effect );
}
