/**
 * @file sample.cpp
 * Coding style sample (this first line is the brief description).
 *
 * Our complete coding standard is at
 * http://www.acm.uiuc.edu/gamebuilders/services/projects/CodingStyle.txt
 *
 * This file hopes to demonstrate all of our coding style guidelines.
 * Notice the Doxygen comments.  We're using the JavaDoc Doxygen style,
 * which means all doc-comments start with a double star, and special
 * commands start with an @.  All our files must contain a comment like
 * this one, with the file command at the top. (This paragraph, ie
 * everything that follows the brief description, is the detailed
 * description).
 */

#include "sample.h"

/**
 * Global Function Demo (brief)
 *
 * This demonstrates a global function, and additionally, how to document
 * functions. (Detailed).
 * 
 * @param x the first parameter's documentation
 * @param y the second parameter's documentation
 * @return description of the return
 */
char *sample_func(char x, int y)
{
    return "hi";
}

/**
 * A static method.
 *
 * This one happens to return nothing and accept nothing.
 */
void Sample::static_func()
{
}

/**
 * Default Constructor.
 *
 * Creates an empty container and sets our variable to 0.  Initializer
 * lists should be formatted like this (colon on the prototype line, with
 * initializers starting on the next line).
 */
Sample::Sample() :
    _super_private_var(0),
    _contents(0)
{
}

/**
 * Our style demonstrator.
 * 
 * This will attempt to use most C/C++ constructs to demonstrate how they
 * should be formatted.  Things to notice: 4 space indent, {} placement, ()
 * spacing, case labels on same indent level of switch.
 *
 * @param n some parameter
 * @return a number of some sort
 */
int Sample::demonstrate_style(int n)
{
    // declare all variables at the beginning
    // (question: even for-loop iterators?)
    Triple elem;
    int retval = 0;
    int i;

    // flow control statements should have a space before the (.
    switch ( n ) {
    case 1:
    case 2:
    case 3:
        elem.one = n;
        elem.two = n * 2;
        elem.three = n * 3;
        // function calls should NOT have a space before the (.
        _contents.push_back( elem );
        break;
    default:
        for ( i = 0; i < n; i++ ) {
            elem.one = i;
            elem.two = ( i + 1 ) * 2;
            elem.three = _private_func( elem.one, elem.two );
            _contents.push_back( elem );
        }
        break;
    }

    // use braces even if the body is only one line.
    // note that open braces are on the same line as the flow control.
    if ( n < 0 ) {
        retval = -1;
    }
    else if ( n == 0 ) {
        retval = 0;
    }
    else {
        retval = n;
    }

    while ( ! _super_private_var ) {
        _super_private_var += 1;
    }

    return retval;
}

/**
 * A private function.
 *
 * Just a private function.  It does some math.
 *
 * @param x first integer
 * @param y second integer
 * @return the sum of the arguments, times my super private variable
 */
int Sample::_private_func(int x, int y)
{
    return ( x + y ) * _super_private_var;
}

