#include "gui.h"

GUI::GUI( std::string title, int width, int height )
{
    // there are weird seg-fault issues with this line on my MSVC6 + XP box,
    // tracked down to the EnumDevices call in CL_DisplayWindow_Win32::create_new_window
    _window = new CL_DisplayWindow( title, width, height );

    _gui_manager = NULL;

    _style = NULL;

    _component_manager = NULL;

    _resource_manager = NULL;

    _gamestate = NULL;

    _quit = false;

    // this hack-ish code is all because I dont know how to use an
    // initializer list on a private member variable...

    // the first member of the struct needs to match the name value of the
    // window in the xml file while the second member needs to be the
    // corresponding game_state.  To be able to use game_state->current_game_state
    // as an index into this array THE MENU ORDER MUST MATCH THE ORDER OF
    // THE STATE ENUM IN GAMESTATE.H
    MenuName temp_menu_names[] =
    {
        { "Main",                MAIN },
        { "Level Loading",       LEVEL_LOADING },
        { "GamePlay",            GAMEPLAY },
        { "End Level",           END_LEVEL },
        { "End Game",            END_GAME },
        { "Level Select",        LEVEL_SELECT },
        { "Ars Physica Help",    HELP },
        { "Options Video Sound", OPTIONS_VIDEO_SOUND },
        { "Options Controls",    OPTIONS_CONTROLS },
        { "Esc Dialog",          ESC_DIALOG },
        { "Run Sim",             RUN_SIM }
    };

    // count only works because its a compile-time variable!
    _num_menus = sizeof( temp_menu_names ) / sizeof( *temp_menu_names );

    _menu_names = (MenuName*) malloc( _num_menus * sizeof( MenuName ) );

    memcpy( _menu_names, temp_menu_names, sizeof( temp_menu_names ) );

    _slot_gui_window_close = new CL_Slot[ _num_menus ];
    _slot_gui_window_move = new CL_Slot[ _num_menus ];
    _window_rects = new CL_Rect[ _num_menus ];

    // the first member of the struct needs to match the name value of the
    // various buttons in the xml file while the second member needs to be
    // the corresponding handler
    ButtonHandler temp_button_handlers[] =
    {
        { "StartGame",       &GUI::on_start_game },
        { "SelectLevel",     &GUI::on_select_level },
        { "Options",         &GUI::on_options },
        { "StartLevel",      &GUI::on_start_level },
        { "Help",            &GUI::on_help },
        { "ReturnMainMenu",  &GUI::on_return_main_menu },
        { "LevelLoadOK",     &GUI::on_level_load },
        { "HelpOK",          &GUI::on_help_ok },
        { "MainQuit",        &GUI::on_quit },
        { "EscQuit",         &GUI::on_quit },
        { "NewLevel",        &GUI::on_select_level },
        { "ChangeOptions",   &GUI::on_change_options },
        { "SetOptions",      &GUI::on_options_ok },
        { "ControlOptions",  &GUI::on_control_options },
        { "SetControls",     &GUI::on_set_controls },
        { "NewLevel",        &GUI::on_new_level },
        { "EscMainMenu",     &GUI::on_esc_main_menu },
        { "StartSim",        &GUI::on_start_sim },
        { "ResetSim",        &GUI::on_reset_sim },
        { "ResetLevel",      &GUI::on_reset_level },
        { "NextLevel",       &GUI::on_next_level }
    };

    _num_buttons = sizeof( temp_button_handlers ) / sizeof( *temp_button_handlers );

    _button_handlers = (ButtonHandler*) malloc( _num_buttons * sizeof( ButtonHandler ) );

    memcpy( _button_handlers, temp_button_handlers, sizeof( temp_button_handlers ) );


    _slot_button = new CL_Slot[ _num_buttons ];

    _ignore_move_signal = _window_moved = false;

    _show_esc_dialog = false;
}

GUI::~GUI( void )
{

}

int GUI::add_resources( std::string resource_name )
{
    if( _resource_manager == NULL )
    {
        if( ( _resource_manager = new CL_ResourceManager( resource_name ) ) == NULL )
        {
            return 0;
        }
    }
    else
    {
        _resource_manager->add_resources( CL_ResourceManager( resource_name ) );
    }

    return 1;
}

int GUI::initialize( GameState* gamestate )
{
    if( _resource_manager == NULL )
        return -1;

    if( ( _style = new CL_StyleManager_Silver( _resource_manager ) ) == NULL )
        return -1;

    if( ( _gui_manager = new CL_GUIManager( _style ) ) == NULL )
        return -1;

    if( ( _component_manager = new CL_ComponentManager( "gui", _resource_manager, _gui_manager ) ) == NULL )
        return -1;

    // by default, no menus should be visible
    for( int i = 0; i < _num_menus; i++ )
    {
        ( (CL_Window *) _component_manager->get_component( _menu_names[ i ].name ) )->show( false );

        // get the original position of this window so we can move it back if the
        // user tries to manually move it
        _window_rects[ i ] = ((CL_Window *) _component_manager->get_component( _menu_names[ i ].name ) )->get_position();
    }

    // special handling for GAMEPLAY and RUN_SIM states since they're made up
    // of buttons and not windows
    ((CL_Window *) _component_manager->get_component( "StartSim" ) )->show( false );
    ((CL_Window *) _component_manager->get_component( "ResetSim" ) )->show( false );
    ((CL_Window *) _component_manager->get_component( "ResetLevel" ) )->show( false );

    if( ( _gamestate = gamestate ) == NULL )
        return -1;

    update_menus();

    return 0;
}

int GUI::register_signals( void )
{
    if( _component_manager == NULL || _gui_manager == NULL || _window == NULL )
        return -1;

    // register every button on every button with their respective handlers
    for( int i = 0; i < _num_buttons; i++ )
    {
        _slot_button[ i ] = ( (CL_Button *) _component_manager->
                get_component( _button_handlers[ i ].name ) )->sig_clicked().connect(
                this, _button_handlers[ i ].handler );
    }

    // register all the window close and move signals with two pseudo-dummy handlers
    for( int j = 0; j < _num_menus; j++ )
    {
        _slot_gui_window_close[ j ] = ( (CL_Window *) _component_manager->
                get_component( _menu_names[ j ].name ) )->sig_close().connect_virtual(
                this, &GUI::on_gui_window_close );

        _slot_gui_window_move[ j ] = ( (CL_Window *) _component_manager->
                get_component( _menu_names[ j ].name ) )->sig_move().connect(
                this, &GUI::on_gui_window_move );
    }

    // do any "beginning of frame" processing
    _slot_paint = _gui_manager->sig_paint().connect( this, &GUI::on_paint );

    // only allow the application to be closed by closing the actual OS window
    _slot_quit = _window->sig_window_close().connect( this, &GUI::on_main_window_close );

    // watch for key presses
    _slot_key_press = CL_Keyboard::sig_key_down().connect( this, &GUI::on_key_press );

    return 0;
}

void GUI::on_paint( void )
{
    // CL_Display::clear( CL_Color( 150, 150, 150 ) );
}

void GUI::on_main_window_close( void )
{
    _quit = true;
}

void GUI::on_key_press( const CL_InputEvent &key )
{
    if( _gamestate == NULL )
        return;

    switch( key.id )
    {
        case CL_KEY_ESCAPE:

        if( _gamestate->get_current_state() == GAMEPLAY ||
                _gamestate->get_current_state() == ESC_DIALOG )
        {
            toggle_esc_dialog();
        }

        break;
    }
}

void GUI::toggle_esc_dialog( void )
{
    if( _gamestate == NULL )
        return;

    _show_esc_dialog = !_show_esc_dialog;

    _gamestate->change_state( _show_esc_dialog ? ESC_DIALOG : GAMEPLAY );

    update_menus();
}

void GUI::on_gui_window_close( CL_SlotParent_v0 &parent_handler )
{
    // use connect virtual so this message isnt passed any higher, and since
    // this function effectively does nothing the effect is that GUI windows
    // can't be manually closed
}

void GUI::on_gui_window_move( int x, int y )
{
    if( _ignore_move_signal )
        return;

    // can't do the actual positioning back to the original location in this
    // function otherwise we enter an infinite loop and crash n' burn
    _window_moved = true;
}

void GUI::on_start_game( void )
{
    if( _component_manager == NULL || _gamestate == NULL )
        return;

    cout << "Starting Game..." << endl;

    // move actual filename out to a #define or static or global resource file
    _gamestate->load_level( "resources/levels/level1.xml" );

    ((CL_Label *) _component_manager->get_component( "LevelLoadingLabel" ) )->set_text( _gamestate->get_level()->get_description() );

    _gamestate->change_state( LEVEL_LOADING );

    update_menus();
}

void GUI::on_select_level( void )
{
    if( _component_manager == NULL || _gamestate == NULL )
        return;

    cout << "Selecting Level..." << endl;

    _gamestate->change_state( LEVEL_SELECT );

    // clear out the list box of previous entries and reload the filenames
    // (allows for adding new levels while AP is running)
    ((CL_ListBox *) _component_manager->get_component( "LevelList" ) )->clear();

    CL_DirectoryScanner scanner;

    // find all files of the form "level*.xml" (case-insensitive)
    if( scanner.scan( "resources/levels/", "level*.xml" ) )
    {
        while( scanner.next() )
        {
            std::string temp = scanner.get_name();

            // get rid of the ".xml" extension
            temp.erase( temp.find_first_of( ".", 0 ), temp.size() );

            ((CL_ListBox *) _component_manager->get_component( "LevelList" ) )->insert_item( temp );
        }
    }

    update_menus();
}

void GUI::on_start_level( void )
{
    if( _component_manager == NULL || _gamestate == NULL )
        return;

    // nothing's been selected
    if( ((CL_ListBox *) _component_manager->get_component( "LevelList" ) )->get_current_item() == -1 )
        return;

    std::string selected_level = ((CL_ListBox *) _component_manager->get_component( "LevelList" ) )->get_current_text();

    cout << "Selected Level: " << selected_level << endl;

    // reformat text back into a filename
    selected_level.insert(0, "resources/levels/" );
    selected_level.append( ".xml" );

    _gamestate->load_level( selected_level );

    ((CL_Label *) _component_manager->get_component( "LevelLoadingLabel" ) )->set_text( _gamestate->get_level()->get_description() );

    _gamestate->change_state( LEVEL_LOADING );

    update_menus();
}

void GUI::on_help( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Displaying Help..." << endl;

    _gamestate->change_state( HELP );

    update_menus();
}

void GUI::on_return_main_menu( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Returning to Main Menu..." << endl;

    _gamestate->change_state( MAIN );

    update_menus();
}

void GUI::on_level_load( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Starting Level..." << endl;

    _gamestate->change_state( GAMEPLAY );

    update_menus();
}

void GUI::on_help_ok( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Returning to Main Menu..." << endl;

    _gamestate->change_state( MAIN );

    update_menus();
}

void GUI::on_quit( void )
{
    cout << "Quitting..." << endl;

    _quit = true;
}

void GUI::on_esc_main_menu( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Returning to Main menu..." << endl;

    toggle_esc_dialog();

    _gamestate->change_state( MAIN );

    update_menus();
}

void GUI::on_new_level( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Choosing New Level..." << endl;

    toggle_esc_dialog();

    _gamestate->change_state( LEVEL_SELECT );

    update_menus();
}

void GUI::on_change_options( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Changing Options..." << endl;

    toggle_esc_dialog();

    _gamestate->change_state( OPTIONS_VIDEO_SOUND );

    update_menus();
}

void GUI::on_options( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Changing Options..." << endl;

    _gamestate->change_state( OPTIONS_VIDEO_SOUND );

    update_menus();
}

void GUI::on_options_ok( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Setting Options..." << endl;

    // set all selected options here

    if( _gamestate->get_previous_state() == GAMEPLAY )
    {
        _gamestate->change_state( GAMEPLAY );
    }
    else
    {
        _gamestate->change_state( MAIN );
    }

    update_menus();
}

void GUI::on_control_options( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Changing Control Options..." << endl;

    // together with the next function, yet another small hack so that we
    // can keep track of where we came from in this doubly-nested menu
    State temp = _gamestate->get_previous_state();

    _gamestate->change_state( OPTIONS_CONTROLS );

    update_menus();

    _gamestate->set_previous_state( temp );
}

void GUI::on_set_controls( void )
{
    if( _gamestate == NULL )
        return;

    cout << "Setting Control Options..." << endl;

    // set control options here

    State temp = _gamestate->get_previous_state();

    _gamestate->change_state( OPTIONS_VIDEO_SOUND );

    update_menus();

    _gamestate->set_previous_state( temp );
}

void GUI::on_start_sim( void )
{
    if( _gamestate == NULL )
        return;

    _gamestate->change_state( RUN_SIM );

    update_menus();
}

void GUI::on_reset_sim( void )
{
    if( _gamestate == NULL )
        return;

    _gamestate->reset_sim();

    update_menus();
}

void GUI::on_reset_level( void )
{
    if( _gamestate == NULL )
        return;

    _gamestate->reset_level();

    update_menus();
}

void GUI::on_next_level( void )
{
    if( _gamestate == NULL )
        return;

    string next_level( "level1.xml" );

    // populate next_level with name of actual next level

    _gamestate->load_level( next_level );

    update_menus();
}

void GUI::update_menus( void )
{
    if( _component_manager == NULL || _gamestate == NULL )
        return;

    State current_state = _gamestate->get_current_state();
    State previous_state = _gamestate->get_previous_state();

    // hide the previous menu and show the current one
    ((CL_Window *) _component_manager->get_component( _menu_names[ previous_state ].name ) )->show( false );
    ((CL_Window *) _component_manager->get_component( _menu_names[ current_state ].name ) )->show( true );

    // special handling for GAMEPLAY/RUN_SIM GameState (windows shouldn't be visible
    // and actual GUI elements are just buttons)
    if( current_state == GAMEPLAY || current_state == RUN_SIM )
    {
        ((CL_Window *) _component_manager->get_component( _menu_names[ current_state ].name ) )->show( false );

        ((CL_Window *) _component_manager->get_component( "StartSim" ) )->show( current_state == GAMEPLAY ? true : false );
        ((CL_Window *) _component_manager->get_component( "ResetSim" ) )->show( current_state == GAMEPLAY ? false : true );
        ((CL_Window *) _component_manager->get_component( "ResetLevel" ) )->show( true );
    }
    else if( previous_state == GAMEPLAY || previous_state == RUN_SIM )
    {
        ((CL_Window *) _component_manager->get_component( "StartSim" ) )->show( false );
        ((CL_Window *) _component_manager->get_component( "ResetSim" ) )->show( false );
        ((CL_Window *) _component_manager->get_component( "ResetLevel" ) )->show( false );
    }
}

int GUI::render( void )
{
    if( _component_manager == NULL || _gui_manager == NULL || _gamestate == NULL )
        return 0;

    // more hack-ish code because I don't know how to set the windows to be
    // stationary in a more elegant fashion
    if( _window_moved )
    {
        State current_state = _gamestate->get_current_state();

        // use ignore_move_signal to keep us out of an infinite loop
        _ignore_move_signal = true;

        ((CL_Window *) _component_manager->get_component( _menu_names[ current_state ].name ) )->set_position( _window_rects[ current_state ].left, _window_rects[ current_state ].top );

        _ignore_move_signal = false;
        _window_moved = false;
    }

    // get changes made by other parts of the system since the last frame
    update_menus();

    // HACK: clanlib should handle this part
    glPushAttrib(GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);
    _gui_manager->show();
    glPopAttrib();

    return 1;
}

CL_GraphicContext* GUI::get_gc( void )
{
    if( _window == NULL )
        return NULL;

    return _window->get_gc();
}

bool GUI::quit( void )
{
    return _quit;
}
