#ifndef _GUI_H
#define _GUI_H

#include <ClanLib/core.h>
#include <ClanLib/gui.h>
#include <ClanLib/gl.h>
#include <ClanLib/guistylesilver.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>

#include "gamestate.h"

class GUI;

typedef void (GUI::*callback)();

struct ButtonHandler { const char* name; callback handler; };
struct MenuName { const char* name; State game_state; };

class GUI {
public:

    GUI( std::string title, int width, int height );

    ~GUI( void );

    int add_resources( std::string resource_name );

    int initialize( GameState* gamestate );

    int register_signals( void );

    int render( void );

    CL_GraphicContext* get_gc( void );

    bool quit( void );

private:

    void update_menus( void );

    void on_paint( void );
    void on_main_window_close( void );
    void on_gui_window_close( CL_SlotParent_v0 &parent_handler );
    void on_gui_window_move( int x, int y );
    void on_key_press( const CL_InputEvent &key );


    void on_start_game( void );
    void on_select_level( void );
    void on_start_level( void );
    void on_help( void );
    void on_return_main_menu( void );
    void on_level_load( void );
    void on_help_ok( void );
    void on_quit( void );
    void on_change_options( void );
    void on_options( void );
    void on_options_ok( void );
    void on_control_options( void );
    void on_set_controls( void );
    void on_new_level( void );
    void on_esc_main_menu( void );
    void on_start_sim( void );
    void on_reset_sim( void );
    void on_reset_level( void );
    void on_next_level( void );

    void toggle_esc_dialog( void );


    CL_DisplayWindow* _window;

    CL_GUIManager* _gui_manager;

    CL_StyleManager_Silver* _style;

    CL_ComponentManager* _component_manager;

    CL_ResourceManager* _resource_manager;


    CL_Slot* _slot_button;
    CL_Slot* _slot_gui_window_close;
    CL_Slot* _slot_gui_window_move;

    CL_Rect* _window_rects;

    CL_Slot _slot_paint;

    CL_Slot _slot_quit;

    CL_Slot _slot_key_press;


    GameState* _gamestate;

    bool _quit;

    bool _window_moved, _ignore_move_signal;

    bool _show_esc_dialog;


    int _num_menus, _num_buttons;


    ButtonHandler* _button_handlers;

    MenuName* _menu_names;
};

#endif
