#include <ClanLib/core.h>
#include <ClanLib/gui.h>
#include <ClanLib/gl.h>
#include <ClanLib/guistylesilver.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>

#include <iostream>
using namespace std;

#include "gamestate.h"
#include "gui.h"
#include "scene.h"

class AP : public CL_ClanApplication
{
public:
    virtual int main(int argc, char** argv)
    {
        // Create a console window for text-output if not available
        CL_ConsoleWindow console("Console", 80, 10000 );
        console.redirect_stdio();

        try {
            cout << "Initializing ClanLib..." << endl;

            CL_SetupCore::init();
            CL_SetupDisplay::init();
            CL_SetupGL::init();
            CL_SetupGUI::init();

            cout << "Creating GameState..." << endl;
            
            game_state = new GameState();

            cout << "Creating GUI..." << endl;

            gui = new GUI( "AP GUI", 800, 600 );

            cout << "Adding Resources..." << endl;

            if( !gui->add_resources( "resources/gui/GUIStyleSilver/gui.xml" ) )
                return 0;

            if( !gui->add_resources( "resources/gui/gui.xml" ) )
                return 0;

            cout << "Initializing GUI..." << endl;

            if( gui->initialize( game_state ) < 0 )
                return 0;

            cout << "Registering GUI signals..." << endl;

            if( gui->register_signals() < 0 )
                return 0;

            cout << "Creating SceneManager..." << endl;

            scene_manager = new SceneManager( gui->get_gc(), game_state );

            cout << "Registering SceneManager signals..." << endl;

            if( scene_manager->register_signals() < 0 )
                return 0;

            /*
            camera = new Camera( Vec3f( 0.0f, 0.0f, -6.0f ), Vec3f( 0.0f, 0.0f, 6.0f ), Vec3f( 0.0f, 1.0f, 0.0f ),
            60.0f, 1.0f, 10000.0f, width, height, PERSPECTIVE );
            */

            /*
            camera = new SphericalCamera( Vec3f( 0.0f, 0.0f, 0.0f ), 6.0f, M_PI/2, 0,
            60.0f, 1.0f, 10000.0f, width, height, PERSPECTIVE );
            */

            cout << "Entering Game Loop..." << endl;

            last_tick = CL_System::get_time();

            // add a floor plane for testing
            dGeomID planeID = dCreatePlane( get_space_id(), 0, 1, 0, -20 );


            while( !gui->quit() )
            {
                game_state->tick();

                cur_tick = CL_System::get_time();
                double t = double(cur_tick - last_tick) / 1000.0; 

                scene_manager->tick( t );

                last_tick = cur_tick;

                scene_manager->render();

                gui->render();

                CL_Display::flip();
                CL_System::keep_alive( 10 );
            }

            dCloseODE();

            CL_SetupDisplay::deinit();
            CL_SetupGL::deinit();
            CL_SetupGUI::deinit();
            CL_SetupCore::deinit();
        }
        catch (CL_Error e)
        {
            std::cout << e.message.c_str() << std::endl;

            // Display console close message and wait for a key
            console.display_close_message();
        }

        return 0;
    }

private:

    GameState* game_state;

    GUI* gui;

    SceneManager* scene_manager;

    int last_tick, cur_tick;

} app;
