#ifndef _CAMERA_H
#define _CAMERA_H

#include "main.h"

#include <ClanLib/display.h>

#include <gfx/vec3.h>

/**
 * A camera that always looks at the origin.
 *
 * Note: not quite spherical coords, because phi is usually the angle from
 * the vertical axis, whereas my equivalent "elevation" is the angle from
 * the horizontal plane (basically, 90 - phi).  Also, all the axes are
 * different (for ex, y is up here, not z).
 */
class Camera {
public:

    /// different types of projections the camera supports
    enum ProjectType {
        ORTHOGRAPHIC,
        PERSPECTIVE
    };

    Camera(ProjectType pt = PERSPECTIVE);

    double get_radius() { return _radius; }

    Vec3 get_position() const;

    Vec3 get_right_vec();
    Vec3 get_up_vec();
    Vec3 get_forward_vec();

    void set_radius( const double radius ) { _radius = radius; }
    void set_elevation( const double elevation ) { _elevation = elevation; }
    void set_azimuth( const double azimuth ) { _azimuth = azimuth; }

    /**
     * Apply the camera's viewport and projection transformations.
     */
    void apply_projection();

    /**
     * Apply the camera's modelview transformations.
     */
    void apply_view();

protected:

    double _radius;          ///< distance of camera from origin
    double _elevation;       ///< vertical angle (phi in spherical coords)
    double _azimuth;         ///< horizontal angle (theta)

    ProjectType _project_type;   ///< orthographic or perspective

    friend std::ostream& operator<<(std::ostream & os, const Camera& camera);
    friend void mouse_drag_camera(Camera *camera, Vec2 prev, Vec2 cur);
};

#endif // header guard
