#include "object.h"
#include "scene.h"

#include <ClanLib/display.h>

void mouse_drag_object(Camera *camera, Object *obj, Vec2 prev, Vec2 cur)
{
    Vec3 right_v = camera->get_right_vec();
    Vec3 forward_v = camera->get_forward_vec();
    Vec2 drag = (cur - prev) * camera->get_radius() / 700.0;

    // project into the xz plane (drop the y component)
    right_v[1] = 0;
    unitize( right_v );

    forward_v[1] = 0;
    unitize( forward_v );

    if ( CL_Keyboard::get_keycode( CL_KEY_CONTROL ) ) {
        if ( CL_Mouse::get_keycode( CL_MOUSE_LEFT ) ) {

            float theta_x = (float) ( cur[ 0 ] - prev[ 0 ] ) / CL_Display::get_width() * M_PI;

            float theta_y = (float) ( cur[ 1 ] - prev[ 1 ] ) / CL_Display::get_height() * M_PI;

            dQuaternion quat;
            dRealPtrTodQuaternion( quat, dBodyGetQuaternion( obj->get_body_id() ) );
            
            Quat rotate = axis_to_quat( -camera->get_right_vec(), -theta_y ) * 
                          axis_to_quat( cross( camera->get_right_vec(), camera->get_forward_vec() ), -theta_x ) * 
                          Quat( quat[ 0 ], quat[ 1 ], quat[ 2 ], quat[ 3 ] );

            dQuaternion result = { rotate.vector()[ 0 ], rotate.vector()[ 1 ], rotate.vector()[ 2 ], rotate.scalar() };

            obj->set_orientation_quat( result );
        }
    }
    else {
        // translation
        if ( CL_Mouse::get_keycode( CL_MOUSE_LEFT ) ) {
            // drag in xz plane
            Vec3 dx = drag[0] * right_v;
            Vec3 dy = -drag[1] * forward_v;

            obj->set_position( obj->get_position() + dx + dy );
        }
        else if ( CL_Mouse::get_keycode( CL_MOUSE_RIGHT ) ) {
            // drag along y axis
            Vec3 dz = -drag[1] * Vec3( 0, 1, 0 );

            obj->set_position( obj->get_position() + dz );
        }
    }
}

void mouse_drag_camera(Camera *camera, Vec2 prev, Vec2 cur)
{
    Vec2 drag = (cur - prev) * camera->get_radius() / 700.0;

    if ( CL_Mouse::get_keycode( CL_MOUSE_LEFT ) ) {
        // rotate about x/y
        camera->_azimuth += -drag[0];
        camera->_elevation += -drag[1];

        if ( camera->_elevation > 90 ) {
            camera->_elevation = 90;
        }
        else if ( camera->_elevation < 0 ) {
            camera->_elevation = 0;
        }
    }
    else if ( CL_Mouse::get_keycode( CL_MOUSE_RIGHT ) ) {
        // zoom in/out
        camera->_radius += drag[1];
    }
}
