#include "material.h"

#include <ClanLib/gl.h>
#include <gfx/gl.h>

Material::Material(Vec4f a, Vec4f d, Vec4f s, Vec4f e, double shiny) :
    ambient(a), diffuse(d), specular(s), emissive(e), shininess(shiny)
{
}

Material::Material(const Material& mat) :
    ambient(mat.ambient), diffuse(mat.diffuse), specular(mat.specular),
    emissive(mat.emissive), shininess(mat.shininess), texture(mat.texture),
    texture_file(mat.texture_file)
{
}

void Material::make_current() const
{
    const GLenum FACE = GL_FRONT;

    // Copy this material's values into the current OpenGL material
    glMaterialfv( FACE, GL_AMBIENT, ambient );
    glMaterialfv( FACE, GL_DIFFUSE, diffuse );
    glMaterialfv( FACE, GL_SPECULAR, specular );
    glMaterialfv( FACE, GL_EMISSION, emissive );
    glMaterialf( FACE, GL_SHININESS, shininess );

    if ( texture > 0 ) {
        glBindTexture( GL_TEXTURE_2D, texture );
    }
}
