/**
 * @file model.cpp
 * Implementation of Model class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#include "model.h"

#include <gfx/gl.h>

Material red(
        Vec4f( 0.45f, 0.025f, 0.025f, 1.0f ),
        Vec4f( 0.45f, 0.025f, 0.025f, 1.0f ),
        Vec4f( 0.45f, 0.025f, 0.025f, 1.0f ),
        Vec4f( 0.0f, 0.0f, 0.0f, 1.0f ),
        30.0f);

Model::Model() :
    _material( red )
{
}

Model::Model(const Material& mat) :
    _material( mat )
{
}

void Model::render()
{
    _material.make_current();

    GLUquadricObj *obj = gluNewQuadric();
    gluSphere(obj, 1.0, 20, 10);
    gluDeleteQuadric(obj);
}

// Draw a wireframe axis-aligned box whose opposite corners are given by the
// points 'min' and 'max'.
void draw_box(const Vec3& min, const Vec3& max, const Vec3f& color)
{
    glPushAttrib(GL_LIGHTING_BIT);
    glDisable(GL_LIGHTING);

    glColor3fv(color);

    glBegin(GL_LINE_LOOP);
    glVertex3d(min[0],min[1],min[2]); glVertex3d(min[0],max[1],min[2]);
    glVertex3d(max[0],max[1],min[2]); glVertex3d(max[0],min[1],min[2]);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glVertex3d(min[0],min[1],max[2]); glVertex3d(min[0],max[1],max[2]);
    glVertex3d(max[0],max[1],max[2]); glVertex3d(max[0],min[1],max[2]);
    glEnd();

    glBegin(GL_LINES);
    glVertex3d(min[0],min[1],min[2]); glVertex3d(min[0],min[1],max[2]);
    glVertex3d(min[0],max[1],min[2]); glVertex3d(min[0],max[1],max[2]);
    glVertex3d(max[0],max[1],min[2]); glVertex3d(max[0],max[1],max[2]);
    glVertex3d(max[0],min[1],min[2]); glVertex3d(max[0],min[1],max[2]);
    glEnd();

    glPopAttrib();
}
