/**
 * @file model.h
 * Header for Model class.
 *
 * Copyright 2003 by GameBuilders.
 * Distributed WITHOUT WARRANTY under the GPL.  See COPYING for details.
 */
#ifndef _MODEL_H
#define _MODEL_H

#include "utils.h"
#include "material.h"

/**
 * Draw a wireframe axis-aligned box whose opposite corners are given by the
 * points 'min' and 'max', in the color 'color'.
 */
void draw_box( const Vec3& min, const Vec3& max, const Vec3f& color );

/**
 * A graphical representation of an object.
 */
class Model {
public:
    Model( void );
    Model( const Material& mat );

    virtual ~Model( void ) {}

    /**
     * Render the object to the screen.
     */
    virtual void render( void );

    /**
     * Draw selection decoration to let the user know we're picked
     */
    virtual void draw_selection( void )
    { draw_box( Vec3( -1.1 ), Vec3( 1.1 ), Vec3f(1.0f, 0.0f, 0.0f) ); }
    
    virtual Vec3 get_min( void ) { return Vec3( 0.0, 0.0, 0.0 ); }
    virtual Vec3 get_max( void ) { return Vec3( 0.0, 0.0, 0.0 ); }
private:
    Material _material; ///< the OpenGL material for this class.
};

#endif // header guard
