#ifndef _SCENE_H
#define _SCENE_H

#include <ClanLib/core.h>
#include <ClanLib/gl.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>

#include <ode/ode.h>

#include "main.h"
#include "actor.h"
#include "object.h"
#include "effects.h"
#include "camera.h"
#include "gamestate.h"

#include <map>
using namespace std;

// since everyone else needs these, need to keep the variables static and the
// associated accessors global
dWorldID get_world_id( void );
dSpaceID get_space_id( void );

class SceneManager {
public:

    SceneManager( CL_GraphicContext* gc, GameState* gamestate );
    ~SceneManager( void );

    /**
     * Elapse t seconds.
     *
     * This calls our our actor's tick functions, then updates all objects'
     * positions.
     */
    void tick( double t );

    /**
     * Render the scene.
     */
    void render( void );

    /**
     * Select the object at screen position pos.
     * _selected_obj contains the picked object, or NULL if background was
     * clicked.
     * @param pos the x,y window coordinates that were selected.
     */
    void select_object( int *pos );

    /**
     * Deselect the currently selected object.
     */
    void deselect_object( void ) { _selected_obj = NULL; }

    Object *get_selected_object( void ) { return _selected_obj; }
    
    int register_signals( void );

private:

    void generate_handlers( void );
    
    void begin_level( void );
    void finish_level( void );

    void on_mouse_down( const CL_InputEvent& event );
    void on_mouse_move( const CL_InputEvent& event );

    static void near_callback( void* data, dGeomID o1, dGeomID o2 );

    CL_Slot _slot_click;
    CL_Slot _slot_unclick;
    CL_Slot _slot_move;

    void gl_init( void );
  
    CL_OpenGLState* _glstate;

    Object *_selected_obj;      ///< currently selected object
    
    Camera _camera;             ///< our camera orientation and position
    
    int _prev_mouse_pos[ 2 ];
    
    int _current_level_id;
};

#endif // header guard
