#ifndef _SPHERICALCAMERA_H
#define _SPHERICALCAMERA_H

#include <gfx/gl.h>
#include <gfx/vec3.h>

#include <iostream>
using namespace std;

#include "fpscamera.h"

/*
 * Derived from Camera, an arcball'ish camera that allows an "inspection"-
 * like view of a scene.  Moves around a sphere with a dynmanic, arbitrary 
 * center point with a dynamic, arbitrary radius.  Using the Camera base class
 * for setting up the projection matrix.
*/

class SphericalCamera : public FPSCamera
{

public:

    SphericalCamera( Vec3f center = Vec3f( 0.0f, 0.0f, 0.0f ), 
            float radius = 100.0f,
            float theta = M_PI/2, float phi = M_PI/4,
            float fov = 45.0f,
            float near_plane = 0.1f, float far_plane = 1000.0f,
            int screen_width = 640, int screen_height = 480,
            ProjType proj_type = PERSPECTIVE );

    ~SphericalCamera( void );

    // bunch of getters and setters...
    float get_radius( void )        {   return radius;  }
    Vec3f get_center( void )        {   return center;  }
    float get_theta( void )     {   return theta;   }
    float get_phi( void )       {   return phi;     }

    void set_radius( float radius ) {   this->radius = radius;  }
    void set_center( Vec3f center ) {   this->center = center;  }
    void set_theta( float theta )   {   this->theta = theta;    }
    void set_phi( float phi )       {   this->phi = phi;        }   

    // we need to overlead these four functions so they do nothing, because the 
    // base class' actionsdon't make sense for a spherical camera
    virtual void rotate_view( float angle );
    virtual void rotate_view( float angle, Vec3f rotate_vec );

    virtual void move( float speed );
    virtual void move( float speed, Vec3f move_vec );

    // Update the camera's position on the sphere
    virtual void mouse_look( Vec2f where, Vec2f last, int which );

    // convert spherical coordinates to a position, forward vector, and
    // up vector and tell opengl where to look
    virtual void look( void );

private:

    Vec3f center;

    float radius;

    float theta;

    float phi;

};

#endif

