#ifndef _UTIL_H
#define _UTIL_H

#include "main.h"

#include <string>
using namespace std;

#include <gfx/mat4.h>

#include <ode/ode.h>

/**
 * Make a rotation matrix out of a triplet of angles to rotate around each
 * axis.
 * Milkshape3D likes to store their rotations this way.
 */
Mat4 rotation_matrix_angles_rad( const Vec3& angles );

// since ODE likes to return float *'s and since C++ doesn't like to return
// arrays we have all these nice conversion functions, the first argument is
// always the target and the second argument is always the source
void Mat4TodMatrix3( dMatrix3 dmat3, const Mat4 mat4 );
void dMatrix3ToMat4( Mat4 mat4, dMatrix3 const dmat3 );
void dMatrix3TodMatrix3( dMatrix3 dmat3_1, const dMatrix3 dmat3_2 );
void dMatrix3ToOpenGL( float oglMat[ 16 ], const dMatrix3 dMat3 );

void dRealPtrTodMatrix4( dMatrix4 dmat4, const float* fltPtr );
void dRealPtrTodMatrix3( dMatrix3 dmat3, const float* fltPtr );
void dRealPtrTodQuaternion( dQuaternion dquat, const float* fltPtr );
void dRealPtrToVec3( Vec3& vec3, const float* fltPtr );

/**
 * Set the translation vectors for a transformation matrix.
 */
void set_translation( Mat4& mat, const Vec3& translate );

/**
 * Invert a translation*rotation matrix.
 */
Mat4 inverse_t_r( const Mat4& mat );

/**
 * @return a GL texture number with filename loaded
 */
uint load_gl_texture( string filename );

#endif // header guard
